@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2017  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2017-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -24,15 +24,17 @@
 
 struct proxy_interface;
 
-#define IWD_ADAPTER_INTERFACE          "net.connman.iwd.Adapter"
-#define IWD_ACCESS_POINT_INTERFACE     "net.connman.iwd.AccessPoint"
-#define IWD_AD_HOC_INTERFACE           "net.connman.iwd.AdHoc"
-#define IWD_AGENT_MANAGER_INTERFACE    "net.connman.iwd.AgentManager"
-#define IWD_DEVICE_INTERFACE           "net.connman.iwd.Device"
-#define IWD_KNOWN_NETWORK_INTREFACE    "net.connman.iwd.KnownNetwork"
-#define IWD_NETWORK_INTERFACE          "net.connman.iwd.Network"
-#define IWD_WSC_INTERFACE              "net.connman.iwd.WiFiSimpleConfiguration"
-#define IWD_STATION_INTERFACE          "net.connman.iwd.Station"
+#define IWD_ADAPTER_INTERFACE            "net.connman.iwd.Adapter"
+#define IWD_ACCESS_POINT_INTERFACE       "net.connman.iwd.AccessPoint"
+#define IWD_AD_HOC_INTERFACE             "net.connman.iwd.AdHoc"
+#define IWD_AGENT_MANAGER_INTERFACE      "net.connman.iwd.AgentManager"
+#define IWD_DEVICE_INTERFACE             "net.connman.iwd.Device"
+#define IWD_KNOWN_NETWORK_INTREFACE      "net.connman.iwd.KnownNetwork"
+#define IWD_NETWORK_INTERFACE            "net.connman.iwd.Network"
+#define IWD_WSC_INTERFACE                "net.connman.iwd.SimpleConfiguration"
+#define IWD_STATION_INTERFACE            "net.connman.iwd.Station"
+#define IWD_STATION_DIAGNOSTIC_INTERFACE "net.connman.iwd.StationDiagnostic"
+#define IWD_AP_DIAGNOSTIC_INTERFACE      "net.connman.iwd.AccessPointDiagnostic"
 
 typedef bool (*proxy_property_match_func_t) (const void *a, const void *b);
 
