@@ -29,10 +29,13 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include <string.h>
+#include <stdlib.h>
+#include <errno.h>
 
 #include "ecc.h"
 #include "ecc-private.h"
 #include "random.h"
+#include "useful.h"
 #include "private.h"
 #include "missing.h"
 
@@ -63,6 +66,7 @@ static const struct l_ecc_curve p256 = {
 	.p = P256_CURVE_P,
 	.n = P256_CURVE_N,
 	.b = P256_CURVE_B,
+	.z = -10,
 };
 
 /*
@@ -96,72 +100,17 @@ static const struct l_ecc_curve p384 = {
 	},
 	.p = P384_CURVE_P,
 	.n = P384_CURVE_N,
-	.b = P384_CURVE_B
+	.b = P384_CURVE_B,
+	.z = -12,
 };
 
 static const struct l_ecc_curve *curves[] = {
-	&p256,
 	&p384,
+	&p256,
 };
 
-LIB_EXPORT const struct l_ecc_curve *l_ecc_curve_get(const char *name)
-{
-	int i;
-
-	if (unlikely(!name))
-		return NULL;
-
-	for (i = 0; curves[i]; i++) {
-		if (!strcmp(curves[i]->name, name))
-			return curves[i];
-	}
-
-	return NULL;
-}
-
-LIB_EXPORT const char *l_ecc_curve_get_name(const struct l_ecc_curve *curve)
-{
-	if (unlikely(!curve))
-		return NULL;
-
-	return curve->name;
-}
-
-LIB_EXPORT size_t l_ecc_curve_get_scalar_bytes(const struct l_ecc_curve *curve)
-{
-	if (unlikely(!curve))
-		return 0;
-
-	return curve->ndigits * 8;
-}
-
-LIB_EXPORT const struct l_ecc_curve *l_ecc_curve_get_ike_group(
-							unsigned int group)
-{
-	unsigned int i;
-
-	for (i = 0; i < L_ARRAY_SIZE(curves); i++) {
-		if (curves[i]->ike_group == group)
-			return curves[i];
-	}
-
-	return NULL;
-}
-
-LIB_EXPORT const struct l_ecc_curve *l_ecc_curve_get_tls_group(
-							unsigned int group)
-{
-	unsigned int i;
-
-	for (i = 0; i < L_ARRAY_SIZE(curves); i++) {
-		if (curves[i]->tls_group == group)
-			return curves[i];
-	}
-
-	return NULL;
-}
-
-LIB_EXPORT const unsigned int *l_ecc_curve_get_supported_ike_groups(void)
+/* Returns supported IKE groups, sorted by the highest effective key size */
+LIB_EXPORT const unsigned int *l_ecc_supported_ike_groups(void)
 {
 	static unsigned int supported_ike_groups[L_ARRAY_SIZE(curves) + 1];
 	static bool ike_first = true;
@@ -179,7 +128,8 @@ LIB_EXPORT const unsigned int *l_ecc_curve_get_supported_ike_groups(void)
 	return supported_ike_groups;
 }
 
-LIB_EXPORT const unsigned int *l_ecc_curve_get_supported_tls_groups(void)
+/* Returns supported TLS groups, sorted by the highest effective key size */
+LIB_EXPORT const unsigned int *l_ecc_supported_tls_groups(void)
 {
 	static unsigned int supported_tls_groups[L_ARRAY_SIZE(curves) + 1];
 	static bool tls_first = true;
@@ -197,6 +147,96 @@ LIB_EXPORT const unsigned int *l_ecc_curve_get_supported_tls_groups(void)
 	return supported_tls_groups;
 }
 
+LIB_EXPORT const struct l_ecc_curve *l_ecc_curve_from_name(const char *name)
+{
+	int i;
+
+	if (unlikely(!name))
+		return NULL;
+
+	for (i = 0; curves[i]; i++) {
+		if (!strcmp(curves[i]->name, name))
+			return curves[i];
+	}
+
+	return NULL;
+}
+
+LIB_EXPORT const struct l_ecc_curve *l_ecc_curve_from_ike_group(
+							unsigned int group)
+{
+	unsigned int i;
+
+	for (i = 0; i < L_ARRAY_SIZE(curves); i++) {
+		if (curves[i]->ike_group == group)
+			return curves[i];
+	}
+
+	return NULL;
+}
+
+LIB_EXPORT const struct l_ecc_curve *l_ecc_curve_from_tls_group(
+							unsigned int group)
+{
+	unsigned int i;
+
+	for (i = 0; i < L_ARRAY_SIZE(curves); i++) {
+		if (curves[i]->tls_group == group)
+			return curves[i];
+	}
+
+	return NULL;
+}
+
+LIB_EXPORT const char *l_ecc_curve_get_name(const struct l_ecc_curve *curve)
+{
+	if (unlikely(!curve))
+		return NULL;
+
+	return curve->name;
+}
+
+LIB_EXPORT unsigned int l_ecc_curve_get_ike_group(
+					const struct l_ecc_curve *curve)
+{
+	if (unlikely(!curve))
+		return 0;
+
+	return curve->ike_group;
+}
+
+LIB_EXPORT unsigned int l_ecc_curve_get_tls_group(
+					const struct l_ecc_curve *curve)
+{
+	if (unlikely(!curve))
+		return 0;
+
+	return curve->tls_group;
+}
+
+LIB_EXPORT struct l_ecc_scalar *l_ecc_curve_get_order(
+						const struct l_ecc_curve *curve)
+{
+	return _ecc_constant_new(curve, curve->n, curve->ndigits * 8);
+}
+
+LIB_EXPORT struct l_ecc_scalar *l_ecc_curve_get_prime(
+						const struct l_ecc_curve *curve)
+{
+	if (unlikely(!curve))
+		return NULL;
+
+	return _ecc_constant_new(curve, curve->p, curve->ndigits * 8);
+}
+
+LIB_EXPORT size_t l_ecc_curve_get_scalar_bytes(const struct l_ecc_curve *curve)
+{
+	if (unlikely(!curve))
+		return 0;
+
+	return curve->ndigits * 8;
+}
+
 static bool ecc_valid_point(struct l_ecc_point *point)
 {
 	const struct l_ecc_curve *curve = point->curve;
@@ -234,7 +274,7 @@ void _ecc_be2native(uint64_t *dest, const uint64_t *bytes,
 							unsigned int ndigits)
 {
 	unsigned int i;
-	uint64_t tmp[L_ECC_MAX_DIGITS];
+	uint64_t tmp[2 * L_ECC_MAX_DIGITS];
 
 	for (i = 0; i < ndigits; i++)
 		tmp[ndigits - 1 - i] = l_get_be64(&bytes[i]);
@@ -273,34 +313,39 @@ static void ecc_compute_y_sqr(const struct l_ecc_curve *curve,
 	memcpy(y_sqr, sum, curve->ndigits * 8);
 }
 
-bool _ecc_compute_y(const struct l_ecc_curve *curve, uint64_t *y, uint64_t *x)
+/*
+ * Compute sqrt(y^2)
+ * Since our prime p satisfies p = 3 (mod 4), we can say:
+ *
+ * y = (y^2)^((p + 1) / 4)
+ *
+ * This avoids the need for a square root function.
+ */
+static void ecc_compute_sqrt(const struct l_ecc_curve *curve,
+					uint64_t *y, const uint64_t *y_sqr)
 {
-	/*
-	 * y = sqrt(x^3 + ax + b) (mod p)
-	 *
-	 * Since our prime p satisfies p = 3 (mod 4), we can say:
-	 *
-	 * y = (x^3 - 3x + b)^((p + 1) / 4)
-	 *
-	 * This avoids the need for a square root function.
-	 */
-
-	uint64_t sum[L_ECC_MAX_DIGITS] = { 0 };
-	uint64_t expo[L_ECC_MAX_DIGITS] = { 0 };
+	uint64_t expo[L_ECC_MAX_DIGITS];
 	uint64_t one[L_ECC_MAX_DIGITS] = { 1ull };
-	uint64_t check[L_ECC_MAX_DIGITS] = { 0 };
 
 	memcpy(expo, curve->p, curve->ndigits * 8);
 
-	/* x^3 - 3x + b */
-	ecc_compute_y_sqr(curve, sum, x);
-
 	/* (p + 1) / 4  == (p >> 2) + 1 */
 	_vli_rshift1(expo, curve->ndigits);
 	_vli_rshift1(expo, curve->ndigits);
 	_vli_mod_add(expo, expo, one, curve->p, curve->ndigits);
 	/* sum ^ ((p + 1) / 4) */
-	_vli_mod_exp(y, sum, expo, curve->p, curve->ndigits);
+	_vli_mod_exp(y, y_sqr, expo, curve->p, curve->ndigits);
+}
+
+bool _ecc_compute_y(const struct l_ecc_curve *curve, uint64_t *y,
+							const uint64_t *x)
+{
+	uint64_t sum[L_ECC_MAX_DIGITS] = { 0 };
+	uint64_t check[L_ECC_MAX_DIGITS] = { 0 };
+
+	/* y = sqrt(x^3 + ax + b) (mod p) */
+	ecc_compute_y_sqr(curve, sum, x);
+	ecc_compute_sqrt(curve, y, sum);
 
 	/* square y to ensure we have a correct value */
 	_vli_mod_mult_fast(check, y, y, curve->p, curve->ndigits);
@@ -400,7 +445,7 @@ void _ecc_point_add(struct l_ecc_point *ret, const struct l_ecc_point *p,
 }
 
 /* result = (base ^ exp) % p */
-void _vli_mod_exp(uint64_t *result, uint64_t *base, uint64_t *exp,
+void _vli_mod_exp(uint64_t *result, const uint64_t *base, const uint64_t *exp,
 			const uint64_t *mod, unsigned int ndigits)
 {
 	unsigned int i;
@@ -427,6 +472,21 @@ void _vli_mod_exp(uint64_t *result, uint64_t *base, uint64_t *exp,
 	memcpy(result, r, ndigits * 8);
 }
 
+__attribute__((noinline)) static int vli_equal(const uint64_t *a,
+							const uint64_t *b,
+							unsigned int ndigits)
+{
+	uint64_t diff = 0;
+	unsigned int i;
+
+	for (i = 0; i < ndigits; i++) {
+		diff |= a[i] ^ b[i];
+		__asm__ ("" : "=r" (diff) : "0" (diff));
+	}
+
+	return (~diff & (diff - 1)) >> 63;
+}
+
 int _vli_legendre(uint64_t *val, const uint64_t *p, unsigned int ndigits)
 {
 	uint64_t tmp[L_ECC_MAX_DIGITS];
@@ -442,12 +502,21 @@ int _vli_legendre(uint64_t *val, const uint64_t *p, unsigned int ndigits)
 
 	if (_vli_cmp(tmp, _1, ndigits) == 0)
 		return 1;
-	else if (_vli_cmp(tmp, _0, ndigits) == 0)
+	if (_vli_cmp(tmp, _0, ndigits) == 0)
 		return 0;
-	else
-		return -1;
+	return -1;
 }
 
+bool _vli_is_zero_or_one(const uint64_t *vli, unsigned int ndigits)
+{
+	uint64_t _1[L_ECC_MAX_DIGITS] = { 1ull };
+	int ret;
+
+	ret = secure_select(vli_equal(vli, _1, ndigits), true, false);
+	ret = secure_select(l_secure_memeq(vli, ndigits * 8, 0), true, ret);
+
+	return ret;
+}
 
 LIB_EXPORT struct l_ecc_point *l_ecc_point_new(const struct l_ecc_curve *curve)
 {
@@ -519,13 +588,148 @@ failed:
 	return NULL;
 }
 
+LIB_EXPORT struct l_ecc_point *l_ecc_point_from_sswu(
+						const struct l_ecc_scalar *u)
+{
+	const struct l_ecc_curve *curve = u->curve;
+	unsigned int ndigits = curve->ndigits;
+	uint64_t z[L_ECC_MAX_DIGITS] = { abs(curve->z) };
+	uint64_t _3[L_ECC_MAX_DIGITS] = { 3ull }; /* -a = 3 */
+	uint64_t u2z[L_ECC_MAX_DIGITS];
+	uint64_t t1[L_ECC_MAX_DIGITS];
+	uint64_t t2[L_ECC_MAX_DIGITS];
+	uint64_t m[L_ECC_MAX_DIGITS];
+	uint64_t t[L_ECC_MAX_DIGITS];
+	uint64_t x1l[L_ECC_MAX_DIGITS];
+	uint64_t x1r[L_ECC_MAX_DIGITS];
+	uint64_t x1[L_ECC_MAX_DIGITS];
+	uint64_t gx1[L_ECC_MAX_DIGITS];
+	uint64_t x2[L_ECC_MAX_DIGITS];
+	uint64_t gx2[L_ECC_MAX_DIGITS];
+	/* reuse m/t/x1l,x1r, they are unused by the time x/v/y/p-y is needed */
+	uint64_t *x = m;
+	uint64_t *v = t;
+	uint64_t *yl = x1l;
+	uint64_t *yr = x1r;
+	bool l;
+	struct l_ecc_point *P;
+
+	/*
+	 * m = (z^2 * u^4 + z * u^2) modulo p
+	 * u2z = u^2 * z
+	 * t2 = u2z^2
+	 * m = t2 - u2z since for all our curves z is negative
+	 */
+	_vli_mod_square_fast(u2z, u->c, curve->p, ndigits);
+	_vli_mod_mult_fast(u2z, u2z, z, curve->p, ndigits);
+	_vli_mod_square_fast(t2, u2z, curve->p, ndigits);
+	_vli_mod_sub(m, t2, u2z, curve->p, ndigits);
+
+	/*
+	 * l = CEQ(m, 0)
+	 * t = inv0(m) where inv0(x) is calculated as x^(p-2) modulo p
+	 */
+	l = l_secure_memeq(m, sizeof(m), 0);
+
+	memset(t2, 0, sizeof(t2));
+	t2[0] = 2ull;
+	_vli_mod_sub(t1, curve->p, t2, curve->p, ndigits);
+	_vli_mod_exp(t, m, t1, curve->p, ndigits);
+
+	/* Calculate: b / z*a, both z and a are negative */
+	_vli_mod_mult_fast(t1, z, _3, curve->p, ndigits);
+	_vli_mod_inv(t1, t1, curve->p, ndigits);
+	_vli_mod_mult_fast(x1l, curve->b, t1, curve->p, ndigits);
+
+	/* t = 1 + t */
+	memset(t2, 0, sizeof(t2));
+	t2[0] = 1ull;
+	_vli_mod_add(t, t, t2, curve->p, ndigits);
+
+	/* t1 = 1 / a */
+	_vli_mod_inv(t1, _3, curve->p, ndigits);
+
+	/* x1r = b * t1 * t */
+	_vli_mod_mult_fast(x1r, curve->b, t1, curve->p, ndigits);
+	_vli_mod_mult_fast(x1r, x1r, t, curve->p, ndigits);
+
+	/* x1 = CSEL(l, (b / (z*a) modulo p), ((-b/a) * (1 + t)) modulo p) */
+	l_secure_select(l, x1l, x1r, x1, ndigits * 8);
+
+	/* gx1 = (x1^3 + a*x1 + b) modulo p */
+	ecc_compute_y_sqr(curve, gx1, x1);
+
+	/* x2 = (z*u^2*x1) modulo p, z is negative, hence the second op */
+	_vli_mod_mult_fast(x2, u2z, x1, curve->p, ndigits);
+	_vli_mod_sub(x2, curve->p, x2, curve->p, ndigits);
+
+	/* gx2 = (x2^3 + a*x2 + b) modulo p */
+	ecc_compute_y_sqr(curve, gx2, x2);
+
+	/*
+	 * l = gx1 is a quadratic residue modulo p
+	 * x is a quadratic residue if x^((p-1)/2) modulo p is zero or one
+	 */
+	_vli_mod_sub(t1, curve->p, t2, curve->p, ndigits);
+	_vli_rshift1(t1, ndigits);
+	_vli_mod_exp(t2, gx1, t1, curve->p, ndigits);
+	l = _vli_is_zero_or_one(t2, ndigits);
+
+	/* v = CSEL(l, gx1, gx2) */
+	l_secure_select(l, gx1, gx2, v, ndigits * 8);
+	/* x = CSEL(l, x1, x2) */
+	l_secure_select(l, x1, x2, x, ndigits * 8);
+	/* y = sqrt(v) */
+	ecc_compute_sqrt(curve, yl, v);
+	/* l = CEQ(LSB(u), LSB(y)) */
+	l = !((u->c[0] & 1ull) ^ (yl[0] & 1ull));
+
+	/* p - y */
+	_vli_mod_sub(yr, curve->p, yl, curve->p, ndigits);
+
+	/* P = CSEL(l, (x,y), (x, p-y)) */
+	P = l_ecc_point_new(curve);
+	memcpy(P->x, x, ndigits * 8);
+	l_secure_select(l, yl, yr, P->y, ndigits * 8);
+
+	return P;
+}
+
+LIB_EXPORT struct l_ecc_point *l_ecc_point_clone(const struct l_ecc_point *p)
+{
+	if (!p)
+		return NULL;
+
+	return l_memdup(p, sizeof(*p));
+}
+
+LIB_EXPORT const struct l_ecc_curve *l_ecc_point_get_curve(
+						const struct l_ecc_point *p)
+{
+	if (!p)
+		return NULL;
+
+	return p->curve;
+}
+
 LIB_EXPORT ssize_t l_ecc_point_get_x(const struct l_ecc_point *p, void *x,
 					size_t xlen)
 {
 	if (xlen < p->curve->ndigits * 8)
 		return -EMSGSIZE;
 
-	_ecc_native2be(x, (uint64_t *) p->x, p->curve->ndigits);
+	_ecc_native2be(x, p->x, p->curve->ndigits);
+
+	return p->curve->ndigits * 8;
+}
+
+LIB_EXPORT ssize_t l_ecc_point_get_y(const struct l_ecc_point *p, void *y,
+					size_t ylen)
+{
+	if (ylen < p->curve->ndigits * 8)
+		return -EMSGSIZE;
+
+	_ecc_native2be(y, p->y, p->curve->ndigits);
 
 	return p->curve->ndigits * 8;
 }
@@ -584,8 +788,90 @@ LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_new(
 	if (!c)
 		return NULL;
 
-	if (buf)
-		_ecc_be2native(c->c, buf, curve->ndigits);
+	if (!buf)
+		return c;
+
+	_ecc_be2native(c->c, buf, curve->ndigits);
+
+	if (!_vli_is_zero_or_one(c->c, curve->ndigits) &&
+			secure_memcmp_64(curve->n, c->c, curve->ndigits) > 0)
+		return c;
+
+	l_ecc_scalar_free(c);
+
+	return NULL;
+}
+
+/*
+ * Build a scalar = value modulo p where p is the prime number for a given
+ * curve.  bytes can contain a numer with up to 2x number of digits as the
+ * curve.  This is used in Hash to Curve calculations.
+ */
+LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_new_modp(
+					const struct l_ecc_curve *curve,
+					const void *bytes, size_t len)
+{
+	struct l_ecc_scalar *c;
+	uint64_t tmp[2 * L_ECC_MAX_DIGITS];
+	unsigned int ndigits = len / 8;
+
+	if (!bytes)
+		return NULL;
+
+	if (len % 8)
+		return NULL;
+
+	if (ndigits > curve->ndigits * 2)
+		return NULL;
+
+	c = _ecc_constant_new(curve, NULL, 0);
+	if (!c)
+		return NULL;
+
+	memset(tmp, 0, sizeof(tmp));
+	_ecc_be2native(tmp, bytes, ndigits);
+
+	_vli_mmod_fast(c->c, tmp, curve->p, curve->ndigits);
+
+	if (!_vli_is_zero_or_one(c->c, curve->ndigits) &&
+			secure_memcmp_64(curve->n, c->c, curve->ndigits) > 0)
+		return c;
+
+	l_ecc_scalar_free(c);
+
+	return NULL;
+}
+
+/*
+ * Takes a buffer of the same size as the curve and scales it to a range
+ * 1..n using value = (value mod (n - 1)) + 1.  For the curves we support
+ * this can be done using a subtraction operation due to the size of n
+ */
+LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_new_reduced_1_to_n(
+					const struct l_ecc_curve *curve,
+					const void *buf, size_t len)
+{
+	uint64_t _1[L_ECC_MAX_DIGITS] = { 1ull };
+	uint64_t tmp[L_ECC_MAX_DIGITS];
+	struct l_ecc_scalar *c;
+
+	if (!buf)
+		return NULL;
+
+	if (len != curve->ndigits * 8)
+		return NULL;
+
+	c = _ecc_constant_new(curve, NULL, 0);
+	if (!c)
+		return NULL;
+
+	_vli_sub(tmp, curve->n, _1, curve->ndigits);
+	_ecc_be2native(c->c, buf, curve->ndigits);
+
+	if (_vli_cmp(c->c, tmp, curve->ndigits) >= 0)
+                _vli_sub(c->c, c->c, tmp, curve->ndigits);
+
+	_vli_add(c->c, c->c, _1, curve->ndigits);
 
 	return c;
 }
@@ -594,12 +880,12 @@ LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_new_random(
 					const struct l_ecc_curve *curve)
 {
 	uint64_t r[L_ECC_MAX_DIGITS];
-	uint64_t zero[L_ECC_MAX_DIGITS] = { 0 };
 
 	l_getrandom(r, curve->ndigits * 8);
 
 	while (_vli_cmp(r, curve->p, curve->ndigits) > 0 ||
-			_vli_cmp(r, zero, curve->ndigits) == 0)
+			_vli_cmp(r, curve->n, curve->ndigits) > 0 ||
+			_vli_is_zero_or_one(r, curve->ndigits))
 		l_getrandom(r, curve->ndigits * 8);
 
 	return _ecc_constant_new(curve, r, curve->ndigits * 8);
@@ -625,12 +911,6 @@ LIB_EXPORT void l_ecc_scalar_free(struct l_ecc_scalar *c)
 	l_free(c);
 }
 
-LIB_EXPORT struct l_ecc_scalar *l_ecc_curve_get_order(
-						const struct l_ecc_curve *curve)
-{
-	return _ecc_constant_new(curve, curve->n, curve->ndigits * 8);
-}
-
 LIB_EXPORT bool l_ecc_scalar_add(struct l_ecc_scalar *ret,
 					const struct l_ecc_scalar *a,
 					const struct l_ecc_scalar *b,
@@ -709,15 +989,6 @@ LIB_EXPORT bool l_ecc_scalar_sum_x(struct l_ecc_scalar *ret,
 	return true;
 }
 
-LIB_EXPORT struct l_ecc_scalar *l_ecc_curve_get_prime(
-						const struct l_ecc_curve *curve)
-{
-	if (unlikely(!curve))
-		return NULL;
-
-	return _ecc_constant_new(curve, curve->p, curve->ndigits * 8);
-}
-
 LIB_EXPORT bool l_ecc_scalars_are_equal(const struct l_ecc_scalar *a,
 						const struct l_ecc_scalar *b)
 {
