@@ -1350,17 +1350,34 @@ static void eapol_send_ptk_3_of_4(struct eapol_sm *sm)
 		key_data_len += gtk_kde[1] + 2;
 	}
 
+	if (sm->handshake->support_ip_allocation &&
+			!sm->handshake->client_ip_addr) {
+		handshake_event(sm->handshake, HANDSHAKE_EVENT_P2P_IP_REQUEST);
+
+		/*
+		 * If .support_ip_allocation was set, the
+		 * HANDSHAKE_EVENT_P2P_IP_REQUEST handler is expected to set
+		 * .client_ip_addr if not already set.  Check if the handler
+		 * was successful in allocating an address, if it wasn't we'll
+		 * just skip the IP Address Allocation KDE.  In either case if
+		 * we need to resend message 3/4 the event callback won't be
+		 * triggered again because the condition above will be false.
+		 */
+		if (!sm->handshake->client_ip_addr)
+			sm->handshake->support_ip_allocation = false;
+	}
+
 	if (sm->handshake->support_ip_allocation) {
 		/* Wi-Fi P2P Technical Specification v1.7 Table 59 */
 		key_data_buf[key_data_len++] = IE_TYPE_VENDOR_SPECIFIC;
 		key_data_buf[key_data_len++] = 4 + 12;
 		l_put_be32(HANDSHAKE_KDE_IP_ADDRESS_ALLOC,
 				key_data_buf + key_data_len + 0);
-		l_put_be32(sm->handshake->client_ip_addr,
+		l_put_u32(sm->handshake->client_ip_addr,
 				key_data_buf + key_data_len + 4);
-		l_put_be32(sm->handshake->subnet_mask,
+		l_put_u32(sm->handshake->subnet_mask,
 				key_data_buf + key_data_len + 8);
-		l_put_be32(sm->handshake->go_ip_addr,
+		l_put_u32(sm->handshake->go_ip_addr,
 				key_data_buf + key_data_len + 12);
 		key_data_len += 4 + 12;
 	}
@@ -1810,9 +1827,9 @@ static void eapol_handle_ptk_3_of_4(struct eapol_sm *sm,
 		hs->support_ip_allocation = ip_alloc_kde != NULL;
 
 		if (ip_alloc_kde) {
-			hs->client_ip_addr = l_get_be32(ip_alloc_kde);
-			hs->subnet_mask = l_get_be32(ip_alloc_kde + 4);
-			hs->go_ip_addr = l_get_be32(ip_alloc_kde + 8);
+			hs->client_ip_addr = l_get_u32(ip_alloc_kde);
+			hs->subnet_mask = l_get_u32(ip_alloc_kde + 4);
+			hs->go_ip_addr = l_get_u32(ip_alloc_kde + 8);
 		} else
 			l_debug("Authenticator ignored our IP Address Request");
 	}
