@@ -212,10 +212,44 @@ enum ie_type {
 	IE_TYPE_DEVICE_LOCATION                      = 204,
 	IE_TYPE_WHITE_SPACE_MAP                      = 205,
 	IE_TYPE_FINE_TIMING_MEASUREMENT_PARAMETERS   = 206,
-	/* Reserved 207 - 220 */
+	IE_TYPE_S1G_OPEN_LOOP_LINK_MARGIN_INDEX      = 207,
+	IE_TYPE_RPS                                  = 208,
+	IE_TYPE_PAGE_SLICE                           = 209,
+	IE_TYPE_AID_REQUEST                          = 210,
+	IE_TYPE_AID_RESPONSE                         = 211,
+	IE_TYPE_S1G_SECTOR_OPERATION                 = 212,
+	IE_TYPE_S1G_BEACON_COMPATIBILITY             = 213,
+	IE_TYPE_SHORT_BEACON_INTERVAL                = 214,
+	IE_TYPE_CHANGE_SEQUENCE                      = 215,
+	IE_TYPE_TWT                                  = 216,
+	IE_TYPE_S1G_CAPABILITIES                     = 217,
+	/* Reserved 218 - 219 */
+	IE_TYPE_SUBCHANNEL_SELECTIVE_TRANSMISSION    = 220,
 	IE_TYPE_VENDOR_SPECIFIC                      = 221,
-	/* Reserved 222 - 254 */
+	IE_TYPE_AUTHENTICATION_CONTROL               = 222,
+	IE_TYPE_TSF_TIMER_ACCURACY                   = 223,
+	IE_TYPE_S1G_RELAY                            = 224,
+	IE_TYPE_REACHABLE_ADDRESS                    = 225,
+	IE_TYPE_S1G_RELAY_DISCOVERY                  = 226,
+	/* Reserved 227 */
+	IE_TYPE_AID_ANNOUNCEMENT                     = 228,
+	IE_TYPE_PV1_PROBE_RESPONSE_OPTION            = 229,
+	IE_TYPE_EL_OPERATION                         = 230,
+	IE_TYPE_SECTORIZED_GROUP_ID_LIST             = 231,
+	IE_TYPE_S1G_OPERATION                        = 232,
+	IE_TYPE_HEADER_COMPRESSION                   = 233,
+	IE_TYPE_SST_OPERATION                        = 234,
+	IE_TYPE_MAD                                  = 235,
+	IE_TYPE_S1G_RELAY_ACTIVATION                 = 236,
+	IE_TYPE_CAG_NUMBER                           = 237,
+	/* Reserved 238 */
+	IE_TYPE_AP_CSN                               = 239,
 	IE_TYPE_FILS_INDICATION                      = 240,
+	IE_TYPE_DILS                                 = 241,
+	IE_TYPE_FRAGMENT                             = 242,
+	/* Reserved 243 */
+	IE_TYPE_RSNX                                 = 244,
+	/* Reserved 245 - 254 */
 	IE_TYPE_EXTENSION                            = 255,
 
 	IE_TYPE_FILS_REQUEST_PARAMETERS              = 256 + 2,
@@ -231,7 +265,23 @@ enum ie_type {
 	IE_TYPE_FILS_PUBLIC_KEY                      = 256 + 12,
 	IE_TYPE_FILS_NONCE                           = 256 + 13,
 	IE_TYPE_FUTURE_CHANNEL_GUIDANCE              = 256 + 14,
+	IE_TYPE_SERVICE_HINT                         = 256 + 15,
+	IE_TYPE_SERVICE_HASH                         = 256 + 16,
+	IE_TYPE_CDMG_CAPABILITIES                    = 256 + 17,
+	IE_TYPE_CLUSTER_PROBE                        = 256 + 21,
+	IE_TYPE_CMMG_CAPABILITIES                    = 256 + 27,
+	IE_TYPE_CMMG_OPERATION                       = 256 + 28,
 	IE_TYPE_OWE_DH_PARAM                         = 256 + 32,
+	IE_TYPE_PASSWORD_IDENTIFIER                  = 256 + 33,
+	IE_TYPE_GLK_GCR_PARAMETER_SET                = 256 + 34,
+	IE_TYPE_VENDOR_SPECIFIC_REQUEST              = 256 + 44,
+	IE_TYPE_MAX_CHANNEL_SWITCH_TIME              = 256 + 52,
+	IE_TYPE_ESTIMATED_SERVICE_PARAMETERS_OUT     = 256 + 53,
+	IE_TYPE_OCI                                  = 256 + 54,
+	IE_TYPE_MSCS_DESCRIPTOR                      = 256 + 88,
+	IE_TYPE_SUPPLEMENTAL_CLASS_2_CAPABILITIES    = 256 + 90,
+	IE_TYPE_REJECTED_GROUPS                      = 256 + 92,
+	IE_TYPE_ANTI_CLOGGING_TOKEN_CONTAINER        = 256 + 93,
 };
 
 /*
@@ -288,6 +338,20 @@ enum ie_rsn_akm_suite {
 	((akm == IE_RSN_AKM_SUITE_SAE_SHA256) || \
 	(akm == IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256))
 
+#define IE_AKM_IS_FT(akm) \
+	(akm & (IE_RSN_AKM_SUITE_FT_OVER_8021X | \
+		IE_RSN_AKM_SUITE_FT_USING_PSK | \
+		IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256 | \
+		IE_RSN_AKM_SUITE_FT_OVER_8021X_SHA384 | \
+		IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA256 | \
+		IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA384))
+
+#define IE_AKM_IS_FILS(akm) \
+	((akm == IE_RSN_AKM_SUITE_FILS_SHA256) || \
+	 (akm == IE_RSN_AKM_SUITE_FILS_SHA384) || \
+	 (akm == IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA256) || \
+	 (akm == IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA384))
+
 #define IE_LEN(ie) \
 	(ie) ? (ie)[1] + 2 : 0
 
@@ -395,6 +459,33 @@ struct ie_neighbor_report_info {
 	bool bss_transition_pref_present : 1;
 };
 
+struct ie_fils_ip_addr_request_info {
+	bool ipv4 : 1;
+	uint32_t ipv4_requested_addr;		/* Zero if none */
+	bool ipv6 : 1;
+	uint8_t ipv6_requested_addr[16];	/* Zero if none */
+	bool dns : 1;
+};
+
+struct ie_fils_ip_addr_response_info {
+	bool response_pending : 1;
+	uint8_t response_timeout;	/* Seconds */
+	uint32_t ipv4_addr;		/* Zero if not provided */
+	uint8_t ipv4_prefix_len;
+	uint32_t ipv4_gateway;		/* Zero if not provided */
+	uint8_t ipv4_gateway_mac[6];
+	uint32_t ipv4_dns;		/* Zero if not provided */
+	uint8_t ipv4_dns_mac[6];	/* Zero if not provided */
+	uint8_t ipv4_lifetime;		/* Zero if not provided */
+	uint8_t ipv6_addr[16];		/* Zero if not provided */
+	uint8_t ipv6_prefix_len;
+	uint8_t ipv6_gateway[16];	/* Zero if not provided */
+	uint8_t ipv6_gateway_mac[6];
+	uint8_t ipv6_dns[16];		/* Zero if not provided */
+	uint8_t ipv6_dns_mac[6];	/* Zero if not provided */
+	uint8_t ipv6_lifetime;		/* Zero if not provided */
+};
+
 extern const unsigned char ieee_oui[3];
 extern const unsigned char microsoft_oui[3];
 extern const unsigned char wifi_alliance_oui[3];
@@ -445,7 +536,7 @@ bool ie_tlv_builder_set_data(struct ie_tlv_builder *builder,
 bool ie_tlv_builder_recurse(struct ie_tlv_builder *builder,
 			struct ie_tlv_builder *recurse);
 unsigned char *ie_tlv_builder_finalize(struct ie_tlv_builder *builder,
-			unsigned int *out_len);
+					size_t *out_len);
 
 uint32_t ie_rsn_cipher_suite_to_cipher(enum ie_rsn_cipher_suite suite);
 
@@ -453,12 +544,15 @@ int ie_parse_rsne(struct ie_tlv_iter *iter, struct ie_rsn_info *info);
 int ie_parse_rsne_from_data(const uint8_t *data, size_t len,
 				struct ie_rsn_info *info);
 bool ie_build_rsne(const struct ie_rsn_info *info, uint8_t *to);
+bool ie_rsne_is_wpa3_personal(const struct ie_rsn_info *info);
 
 int ie_parse_wpa(struct ie_tlv_iter *iter, struct ie_rsn_info *out_info);
 int ie_parse_wpa_from_data(const uint8_t *data, size_t len,
 						struct ie_rsn_info *info);
 bool is_ie_wfa_ie(const uint8_t *data, uint8_t len, uint8_t oi_type);
 bool is_ie_wpa_ie(const uint8_t *data, uint8_t len);
+bool is_ie_default_sae_group_oui(const uint8_t *data, uint16_t len);
+
 bool ie_build_wpa(const struct ie_rsn_info *info, uint8_t *to);
 
 int ie_parse_bss_load(struct ie_tlv_iter *iter, uint16_t *out_sta_count,
@@ -469,19 +563,6 @@ int ie_parse_bss_load_from_data(const uint8_t *data, uint8_t len,
 				uint8_t *out_channel_utilization,
 				uint16_t *out_admission_capacity);
 
-int ie_parse_supported_rates_from_data(const uint8_t *supp_rates_ie,
-					uint8_t supp_rates_len,
-					const uint8_t *ext_supp_rates_ie,
-					uint8_t ext_supp_rates_len,
-					int32_t rssi, uint64_t *data_rate);
-
-int ie_parse_data_rates(const uint8_t *supp_rates_ie,
-			const uint8_t *ext_supp_rates_ie,
-			const uint8_t *ht_ie,
-			const uint8_t *vht_ie,
-			int32_t rssi,
-			uint64_t *data_rate);
-
 int ie_parse_mobility_domain(struct ie_tlv_iter *iter, uint16_t *mdid,
 				bool *ft_over_ds, bool *resource_req);
 int ie_parse_mobility_domain_from_data(const uint8_t *data, uint8_t len,
@@ -527,3 +608,24 @@ int ie_parse_hs20_indication_from_data(const uint8_t *data, size_t len,
 					uint8_t *version, uint16_t *pps_mo_id,
 					uint8_t *domain_id);
 int ie_build_hs20_indication(uint8_t version, uint8_t *to);
+
+enum ie_rsnx_capability {
+	IE_RSNX_PROTECTED_TWT                                = 4,
+	IE_RSNX_SAE_H2E                                      = 5,
+};
+
+bool ie_rsnxe_capable(const uint8_t *rsnxe, unsigned int bit);
+
+int ie_parse_fils_ip_addr_request(struct ie_tlv_iter *iter,
+				struct ie_fils_ip_addr_request_info *out);
+void ie_build_fils_ip_addr_request(
+				const struct ie_fils_ip_addr_request_info *info,
+				uint8_t *to);
+int ie_parse_fils_ip_addr_response(struct ie_tlv_iter *iter,
+				struct ie_fils_ip_addr_response_info *out);
+void ie_build_fils_ip_addr_response(
+			const struct ie_fils_ip_addr_response_info *info,
+			uint8_t *to);
+
+int ie_parse_network_cost(const void *data, size_t len,
+				uint16_t *flags, uint16_t *level);
