@@ -27,6 +27,7 @@ struct netdev;
 struct station;
 enum security;
 struct scan_bss;
+struct scan_freq_set;
 struct network;
 
 enum station_state {
@@ -38,13 +39,22 @@ enum station_state {
 	STATION_STATE_AUTOCONNECT_FULL,
 	/* Connecting */
 	STATION_STATE_CONNECTING,
+	STATION_STATE_CONNECTING_AUTO,
 	STATION_STATE_CONNECTED,
 	STATION_STATE_DISCONNECTING,
 	STATION_STATE_ROAMING
 };
 
+enum station_anqp_state {
+	STATION_ANQP_STARTED,
+	STATION_ANQP_FINISHED,
+};
+
 typedef void (*station_foreach_func_t)(struct station *, void *data);
 typedef void (*station_state_watch_func_t)(enum station_state, void *userdata);
+typedef void (*station_anqp_watch_func_t)(enum station_anqp_state,
+						struct network *network,
+						void *user_data);
 typedef void (*station_destroy_func_t)(void *userdata);
 typedef void (*station_network_foreach_func_t)(struct network *, void *data);
 
@@ -57,6 +67,7 @@ struct network *station_network_find(struct station *station, const char *ssid,
 					enum security security);
 
 void station_set_scan_results(struct station *station, struct l_queue *bss_list,
+				const struct scan_freq_set *freqs,
 				bool add_to_autoconnect);
 
 enum station_state station_get_state(struct station *station);
@@ -66,11 +77,12 @@ uint32_t station_add_state_watch(struct station *station,
 					station_destroy_func_t destroy);
 bool station_remove_state_watch(struct station *station, uint32_t id);
 
-bool station_set_autoconnect(struct station *station, bool autoconnect);
+uint32_t station_add_anqp_watch(station_anqp_watch_func_t func,
+				void *user_data,
+				station_destroy_func_t destroy);
+void station_remove_anqp_watch(uint32_t id);
 
-void station_ap_directed_roam(struct station *station,
-				const struct mmpdu_header *hdr,
-				const void *body, size_t body_len);
+bool station_set_autoconnect(struct station *station, bool autoconnect);
 
 int __station_connect_network(struct station *station, struct network *network,
 				struct scan_bss *bss);
@@ -87,3 +99,5 @@ void station_network_foreach(struct station *station,
 				void *user_data);
 struct l_queue *station_get_bss_list(struct station *station);
 struct scan_bss *station_get_connected_bss(struct station *station);
+
+int station_hide_network(struct station *station, struct network *network);
